(function (VAKS) {
    var $ = jQuery;

    VAKS.ThemeManager = function ()
    {
        this.ui = {
            $loading: $("#vaksShortcodeThemeContainer"),
            $themeSelect: $("[data-vaks-themes]"),
            $overwriteBtn: $("[data-vaks-theme-overwrite]"),
            $createBtn: $("[data-vaks-theme-create]"),
            $deleteBtn: $("[data-vaks-theme-delete]"),
            $exportBtn: $("[data-vaks-theme-export]"),
            $importBtn: $("[data-vaks-theme-import]"),
            modal: {
                $modal: $("[data-vaks-modal]"),
                $title: $("[data-vaks-modal-title]"),
                $primaryBtn: $("[data-vaks-modal-primary-btn]"),
                $secondaryBtn: $("[data-vaks-modal-secondary-btn]"),
                $textarea: $("[data-vaks-modal-textarea]"),
            },
        };

        this.themes = vaksTrans.themes || {};
        this.currentTheme = null;
        this.themeUpdated = function () {};
        this.form = new VAKS.ThemeManagerForm();
        this.form.onSettingsChanged = this.onSettingsChanged.bind(this);
        this.preview = new VAKS.ThemeManagerPreview($("[data-role='widget-preview']"));

        this.onThemeChanged = this.onThemeChanged.bind(this);
        this.overwriteCurrentTheme = this.overwriteCurrentTheme.bind(this);
        this.createTheme = this.createTheme.bind(this);
        this.deleteCurrentTheme = this.deleteCurrentTheme.bind(this);
        this.exportSettings = this.exportSettings.bind(this);
        this.importSettings = this.importSettings.bind(this);

        this.init();
    };

    VAKS.ThemeManager.prototype = {
        init: function ()
        {
            this.ui.$themeSelect.change(this.onThemeChanged);
            this.ui.$overwriteBtn.click(this.overwriteCurrentTheme);
            this.ui.$createBtn.click(this.createTheme);
            this.ui.$deleteBtn.click(this.deleteCurrentTheme);
            this.ui.$exportBtn.click(this.exportSettings);
            this.ui.$importBtn.click(this.importSettings);

            this.currentTheme = this.ui.$themeSelect.val() || null;

            this.refreshUI();
            this.onThemeChanged();
        },

        onSettingsChanged: function (settings) {
            this.preview.triggerReload(settings);
        },

        onThemeChanged: function ()
        {
            this.currentTheme = this.ui.$themeSelect.val();
            this.loadSettings(this.themes[this.currentTheme]);
        },

        loadSettings: function (settings)
        {
            this.form.loadSettings(settings);
            this.refreshUI();
            this.preview.triggerReload(settings);
        },

        updateTheme: function (themeName, settings)
        {
            var self = this;

            this.showLoading();

            $.post({
                url: ajaxurl,
                data: {
                    action: "vaks_theme_update",
                    theme: {
                        name: themeName,
                        settings: settings,
                    },
                    _wpnonce: vaksTrans.nonce,
                },
            })
                .then(function (response) {
                    if (response === "ok")
                    {
                        self.themes[themeName] = settings;
                        self.currentTheme = themeName;
                        self.refreshUI();

                        self.themeUpdated(themeName, settings);
                    }
                    else
                        console.error("Error while updating the theme: "+ response);
                })
                .fail(function (response) {
                    console.error("Error while updating the theme: "+ response.statusText);
                })
                .always(function () {
                    self.hideLoading();
                });
        },

        overwriteCurrentTheme: function (e)
        {
            if (e)
                e.preventDefault();

            if (this.currentTheme === null)
                return ;

            var settings = this.form.getSettings();
            this.updateTheme(this.currentTheme, settings);
        },

        createTheme: function (e)
        {
            if (e)
                e.preventDefault();

            var name = prompt(vaksTrans.enterThemeName);

            if (name)
            {
                var settings = this.form.getSettings();
                this.updateTheme(name, settings);
            }
        },

        deleteCurrentTheme: function (e)
        {
            var self = this;

            this.showLoading();

            $.post({
                url: ajaxurl,
                data: {
                    action: "vaks_theme_delete",
                    theme: this.currentTheme,
                    _wpnonce: vaksTrans.nonce,
                },
            })
                .then(function (response) {
                    if (response === "ok")
                    {
                        delete self.themes[self.currentTheme];
                        self.refreshUI();
                        self.currentTheme = self.ui.$themeSelect.val();
                    }
                    else
                        console.error("Error while deleting the theme: "+ response);
                })
                .fail(function (response) {
                    console.error("Error while deleting the theme: "+ response.statusText);
                })
                .always(function () {
                    self.hideLoading();
                });
        },

        importSettings: function ()
        {
            var modal = this.ui.modal;
            var settings = null;
            var self = this;

            modal.$modal
                .unbind()
                .css("display", "flex")
                .click(function (e) {
                    if (e.target === e.currentTarget)
                        modal.$modal.hide();
                });

            modal.$title.text(vaksTrans.text.pasteThemeInTextarea);

            modal.$primaryBtn
                .unbind()
                .text(vaksTrans.text.import)
                .attr("disabled", true)
                .click(function () {
                    if (settings !== null)
                    {
                        self.loadSettings(settings);
                        modal.$modal.hide();
                    }
                });

            modal.$secondaryBtn
                .unbind()
                .text(vaksTrans.text.close)
                .click(function () {
                    modal.$modal.hide();
                });

            modal.$textarea
                .unbind()
                .val("")
                .select()
                .on("change, input", function () {
                    try {
                        settings = JSON.parse(this.value);
                        modal.$primaryBtn
                            .text(vaksTrans.text.import)
                            .attr("disabled", false);
                    } catch (e) {
                        settings = null;
                        modal.$primaryBtn
                            .text(vaksTrans.text.invalidTheme)
                            .attr("disabled", true);
                    }
                });
        },

        exportSettings: function ()
        {
            var modal = this.ui.modal;

            modal.$modal
                .unbind()
                .css("display", "flex")
                .click(function (e) {
                    if (e.target === e.currentTarget)
                        modal.$modal.hide();
                });

            modal.$title.text(vaksTrans.text.exportTheme);

            modal.$primaryBtn
                .unbind()
                .text(vaksTrans.text.copyToClipboard)
                .attr("disabled", false)
                .click(function () {
                    modal.$textarea.select();
                    document.execCommand("copy");
                    modal.$primaryBtn.text(vaksTrans.text.copied);
                });

            modal.$secondaryBtn
                .unbind()
                .text(vaksTrans.text.close)
                .click(function () {
                    modal.$modal.hide();
                });

            modal.$textarea
                .unbind()
                .val(JSON.stringify(this.form.getSettings(), null, 4))
                .select();
        },

        refreshUI: function ()
        {
            this.updateSelect();
            this.updateButtons();
        },

        updateSelect: function ()
        {
            var self = this;
            var appendOption = function (name, value, selected) {
                var $option = $("<option></option>")
                    .text(name)
                    .val(value)
                    .attr("selected", selected);

                self.ui.$themeSelect.append($option);
            };

            this.ui.$themeSelect.empty();

            for (var name in this.themes)
                appendOption(name, name, (name === this.currentTheme));
        },

        updateButtons: function ()
        {
            this.ui.$overwriteBtn.attr("disabled", (this.currentTheme === null));
            this.ui.$deleteBtn.attr("disabled", this.currentTheme === vaksTrans.defaultThemeName || (this.currentTheme === null));
        },

        showLoading: function ()
        {
            this.ui.$loading.css("display", "flex");
        },

        hideLoading: function ()
        {
            this.ui.$loading.hide();
        },
    };

    new VAKS.ThemeManager();
})(window.VAKS = window.VAKS || {});