<?php
namespace VAKS\Upgrade;

class UpgradeManager
{
    const VERSION_OPTION = "vaks_version";
    const DEFAULT_VERSION = "0.0.0";

    public function bindReportNotice($success, array $report)
    {
        add_action("admin_notices", function () use ($success, $report) {
            require __DIR__ ."/templates/report.php";
        });
    }

    public function upgradeToVersion($toVersion)
    {
        $fromVersion = $this->getOldVersion();
        $scripts = $this->getUpgradeScripts($fromVersion, $toVersion);

        if (count($scripts) > 0)
        {
            $success = $this->applyUpgradeScripts($scripts, $report);

            if ($success)
                $this->updateVersion($toVersion);

            $this->bindReportNotice($success, $report);
        }
        else
            $this->updateVersion($toVersion);
    }

    public function applyUpgradeScripts(array $scripts, &$report)
    {
        $report = array();

        foreach ($scripts as $script)
        {
            $instance = new $script["class"]();

            try {
                $instance->up();
            } catch (\Exception $e) {
                $report[] = array(
                    "status" => "error",
                    "message" => sprintf(__("The upgrade has been interrupted to %s due to an error: %s", "allkeyshop-widget"), "v$script[version]", $e->getMessage()),
                );

                return false;
            }

            $this->updateVersion($script["version"]);

            $report[] = array(
                "status" => "success",
                "message" => sprintf(__("Upgraded to %s", "allkeyshop-widget"), "v$script[version]"),
            );
        }

        return true;
    }

    public function getUpgradeScripts($from, $to)
    {
        $scripts = $this->getAllUpgradeScripts();

        $scripts = array_filter($scripts, function ($script) use ($from, $to) {
            return version_compare($script["version"], $from, ">") && version_compare($script["version"], $to, "<=");
        });

        return $scripts;
    }

    public function getAllUpgradeScripts()
    {
        $path = __DIR__."/Scripts/";
        $entries = scandir($path);
        $scripts = array();

        foreach ($entries as $entry)
        {
            preg_match("#(?<class>v(?<version>\d+_\d+_\d+))\.php#", $entry, $matches);

            if (!empty($matches["version"]) && !empty($matches["class"]))
            {
                $scripts[] = array(
                    "class" => __NAMESPACE__ ."\\Scripts\\$matches[class]",
                    "version" => str_replace("_", ".", $matches["version"]),
                );
            }
        }

        return $scripts;
    }

    public function isUpToDate($toVersion)
    {
        $currentVersion = $this->getOldVersion();
        return version_compare($currentVersion, $toVersion, ">=");
    }

    public function getOldVersion()
    {
        return get_option(self::VERSION_OPTION, self::DEFAULT_VERSION);
    }

    public function updateVersion($version)
    {
        update_option(self::VERSION_OPTION, $version, true);
    }
}